/**
 * Author: Melissa Hovik
 * Last Updated: 11.20.18
 *
 * A basic Chrome extension for overriding new tabs, showing a random puppy from the
 * Stanford Dog API (https://dog.ceo/api/).
 */

(function() {
  "use strict";
  let URL = "https://dog.ceo/api/";
  let BREED_EP = "breed/BREEDNAME/images/random";
  let RANDOM_EP = "breeds/image/random"
  let BREEDS_EP = "breeds/list/all"

  window.addEventListener("load", initialize);
  
  function initialize() {
    fetchRandomPuppy(RANDOM_EP);
    fetchBreedsList();
  }

  function fetchRandomPuppy(endPoint) {
    fetch(URL + endPoint)
      .then(checkStatus)
      .then(JSON.parse)
      .then(showPuppy)
      .catch(handleError);
  }

  function fetchBreedsList() {
    fetch(URL + BREEDS_EP)
      .then(checkStatus)
      .then(JSON.parse)
      .then(populateBreeds)
      .catch(handleError);
  }

  function showPuppy(responseData) {
    $("doggy").src = responseData.message;
    $("doggy").classList.remove("hidden");
    $("results").classList.remove("hidden");
    let selectedBreed = document.querySelector("select").value;
    let text = "You found a";
    if (selectedBreed !== "random") {
      if (startsWithVowel(selectedBreed)) {
        text += "n";
      }
      text += " " + selectedBreed;
    }
    $("title").innerText = text + " puppy!";
  }

  function populateBreeds(responseData) {
    let breedNames = responseData.message;
    for (let breed in breedNames) { 
      let option = document.createElement("option");
      option.innerText = breed.charAt(0).toUpperCase() + breed.substring(1);
      $("puppy-list").appendChild(option);
    }
    $("fetch-btn").disabled = false;
    $("fetch-btn").addEventListener("click", function() {
      let selected = document.querySelector("select").value.toLowerCase();
      if (selected === "random") {
        fetchRandomPuppy(RANDOM_EP);
      } else {
        fetchRandomPuppy(BREED_EP.replace("BREEDNAME", selected));
      }
    });
  }

  function startsWithVowel(word) {
    return word.startsWith("a") || word.startsWith("e") || word.startsWith("i") ||
           word.startsWith("o") || word.startsWith("u");
  }

  function handleError(response) {
    console.log("Unable to contact Dog API");
    $("doggy").src = "mowgli.jpg"; // default puppy image
    $("results").classList.add("hidden");
  }

  /* ==============================  Helper Functions ============================== */
  /*
   * Helper function to return the response's result text if successful, otherwise
   * returns the rejected Promise result with an error status and corresponding text
   * @param {object} response - response to check for success/error
   * @return {object} - valid result text if response was successful, otherwise rejected
   *                    Promise result
   */
  function checkStatus(response) { 
    if (response.status >= 200 && response.status < 300 || response.status == 0) {  
      return response.text();
    } else {  
      return Promise.reject(new Error(response.status + ": " + response.statusText)); 
    } 
  }

  /**
   * Returns the element that has the ID attribute with the specified value.
   * @param {string} id - element ID
   * @return {object} DOM object associated with id.
   */
  function $(id) {
    return document.getElementById(id);
  }
})();
